class GHgpio:
    """GPIO definitions for IoT Greenhouse

    This class defines GPIO pins used for 20 pin P1 connector.
    Used during GPIO configuration.
    """
    RED_LED = 4         #pin 7  GPIO4       P1.1
    WHITE_LED = 14      #pin 8  GPIO14      P1.2
    #GND                pin 9               P1.3
    BUZZER = 15         #pin 10 GPIO15      P1.4     
    AUX1 = 17           #pin 11 GPIO17      P1.5
    SERVO_PWM = 18      #pin 12 GPIO18      P1.6
    SWITCH_TOGGLE = 27  #pin 13 GPIO27      P1.7
    #GND                 pin 14             P1.8
    FAN = 22            #pin 15 GPIO22      P1.9
    AUX2 = 23           #pin 16 GPIO23      P1.10
    #3.3V                pin 17             PI_PWR_LED P1.11
    SWITCH_PB = 24      #pin 18 GPIO24      P1.12
    #ANALOG_DIN         #pin 19 SPI0_MIS I  P1.13
    #GND                 pin 20             P1.14
    #ANALOG_D0UT        #pin 21 SPI0_MIS O  P1.15
    TEMP_SENSOR = 25    #pin 22 GPIO25      P1.16
    #ANALOG_CLK          #pin 23 SPIO_S CLK P1.17
    #ANALOG_CS           #pin 24 SPIO_CE0   P1.18
    #GND                 pin 25             P1.19
    DUAL_LED = 7         #pin 26 GPIO7      P1.20



from time import time
import requests
import json
import ast
 
class GHPostData():
    """Data object for post response
    """
    status_code = 0
    last_update = None
    exception = None

class GHWebService(object):
    """ Web connector for Flow service. Post data using json package.
    """
    POST_DELAY = 30     #Seconds between posts - throttle
    gh_service = None
    url = None
    post_data = None

    def __init__(self, greenhouse_service):
        self.gh_service = greenhouse_service
        self.url = greenhouse_service._url
        
    def post_greenhouse(self):
        #throttle post to every 30 sec
        gh = self.gh_service.greenhouse
        if self.post_data == None or time() > self.post_data.last_update + self.POST_DELAY:
            self.gh_service.update_greenhouse()
            p_data = self._post(gh)
            self.post_data = p_data

               
    def _post(self, gh, verbose=True):
        '''HTTP request post of json dump

        json package formated for Flow service
        '''
        post_data = GHPostData()
        try:
            headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
            if verbose:
                payload = json.dumps(gh.__dict__)
            else:
                payload = {"house_id": gh.house_id, 
                          "group_id": gh.group_id, 
                          "house_number": gh.house_number,
                          "name": gh.name,
                          "temp_inside_F": gh.temp_inside_F,
                          "temp_outside_F": gh.temp_outside_F,
                          "servo_status": gh.servo_status,
                          "fan_status": gh.fan_status,
                          "heater_status": gh.led_white_status,  #white led simulates heater

                          "message": gh.message}

            json_payload = ast.literal_eval(payload)
            r = requests.post(self.url, json=json_payload , headers=headers)
            post_data.status_code = r.status_code
            post_data.last_update = time()
            
        except Exception as ex:
            print("Error: Unable to post data to service. Check for valid Wi-Fi connection. %s" % ex.args[0])
            post_data.exception = ex 
        
        finally:
            return post_data


def test_post():
    from iot_gh.IoTGreenhouseService import IoTGreenhouseService

    ghs = IoTGreenhouseService()
    gh = ghs.greenhouse
    ghs.web_service.post_greenhouse()
    print(gh.post_data.status_code)
    print(gh.post_data.last_update)
    print(gh.post_data.exception)
    
if __name__ == "__main__":
    test_post()


import datetime
import spidev
import pigpio
import configparser, os

from iot_gh.GHgpio import GHgpio
from iot_gh.GHAnalogService import GHAnalogService
from iot_gh.GHBuzzer import GHBuzzer
from iot_gh.GHFan import GHFan
from iot_gh.GHLamps import GHLamps
from iot_gh.GHServo import GHServo
from iot_gh.GHSwitches import GHSwitches
from iot_gh.GHTemperature import GHTemperature
from iot_gh.GHWebService import GHWebService
from iot_gh.IoTGreenhouse import IoTGreenhouse


##debug in VS only
#from spi_mock import spidev_mock as spidev
##from pigpio_mock import pigpio_mock as pigpio
#import pigpio_mock as pigpio

class IoTGreenhouseService(object):
    """IoT Greenhouse service class for IoT Greenhouse
    
    Provides references to input and output objects.
    """
    _pi = None      #pigpio connection
    _url = None
    _servo_ccw_limit = None
    _servo_cw_limit = None
    
    greenhouse = None
    version = None
    
    analog_service = None
    ain_pot = None
    ain_temp = None
    ain_light = None
    ain_aux = None
    buzzer = None
    fan = None
    lamps = None
    servo = None
    switches = None
    temperature = None
    web_service = None

    def __init__(self):

        self._pi = pigpio.pi()             # exit script if no connection
        if not self._pi.connected:
            print("ERROR: unable to connect to pigpio")
            exit()
        self._read_config()
        self._make_components()
        self.greenhouse = self._make_greenhouse("new greenhouse")
        self.update_greenhouse()

    def _read_config(self):
        """Load configuration file from user iot_gh directory.
        """
        try:
            config = configparser.ConfigParser()
            config.read(["iot_gh_system.conf", os.path.expanduser("~/iot_gh/iot_gh_system.conf")],encoding="UTF8")
            self.version = config["IOT_GREENHOUSE"]["VERSION"]
            self._url = config["IOT_GREENHOUSE"]["URL"]
            self._servo_ccw_limit = int(config["IOT_GREENHOUSE"]["SERVO_CCW_LIMIT"])
            self._servo_cw_limit = int(config["IOT_GREENHOUSE"]["SERVO_CW_LIMIT"])
        except Exception as ex:
            raise Exception("Unable to load IoT Greenhouse System Configuration. %s" % ex.args[0])
       
    def _make_components(self):
        """Makes component services for IoT Greenhouse Service.
        """
        spi = spidev.SpiDev()
        self.analog = GHAnalogService(self._pi, spi)
        self.buzzer = GHBuzzer(self._pi, GHgpio.BUZZER)
        self.fan = GHFan(self._pi, GHgpio.FAN)
        self.lamps = GHLamps(self._pi, GHgpio.RED_LED, GHgpio.WHITE_LED, GHgpio.RED_LED, GHgpio.DUAL_LED)
        self.servo = GHServo(self._pi, GHgpio.SERVO_PWM, self._servo_cw_limit, self._servo_ccw_limit)
        self.switches = GHSwitches(self._pi, GHgpio.SWITCH_PB, GHgpio.SWITCH_TOGGLE)
        self.temperature = GHTemperature(self.analog.aux, self.analog.temp)
        self.web_service = GHWebService(self)    
        
    def _make_greenhouse(self, name):
        """Factory class to build IoT Greenhouse data object.
        """
        gh = IoTGreenhouse()
        gh.name = name
        
        #read greenhouse config values
        config = configparser.ConfigParser()
        config.read(["iot_gh.conf", os.path.expanduser("~/iot_gh/iot_gh.conf")],encoding="UTF8")
        gh.house_id = config["IOT_GREENHOUSE"]["HOUSE_ID"]
        gh.group_id = config["IOT_GREENHOUSE"]["GROUP_ID"]
        gh.house_number = config["IOT_GREENHOUSE"]["HOUSE_NUMBER"].zfill(2)
        gh.version = self.version

        return gh
    
    def update_greenhouse(self):
        """Updates IoTGreenhouse object by reading all house 
        states and refreshing gh object.
        """
        try:
    
            self.greenhouse.led_red_state = self.lamps.red.get_state()
            self.greenhouse.led_red_status = self.lamps.red.get_status()
            self.greenhouse.led_white_state = self.lamps.white.get_state()
            self.greenhouse.led_white_status = self.lamps.white.get_status()
            self.greenhouse.led_dual_state =  self.lamps.dual.get_state()
            self.greenhouse.led_dual_status = self.lamps.dual.get_status()
            self.greenhouse.switch_pb_state = self.switches.push_button.get_state()
            self.greenhouse.switch_pb_status = self.switches.push_button.get_status()
            self.greenhouse.switch_toggle_state = self.switches.toggle.get_state()
            self.greenhouse.switch_toggle_status = self.switches.toggle.get_status()
            self.greenhouse.fan_state =  self.fan.get_state()
            self.greenhouse.fan_status =  self.fan.get_status()
            self.greenhouse.servo_position = self.servo.get_value()
            self.greenhouse.servo_status = self.servo.get_status()
            self.greenhouse.heater_state = self.lamps.white.get_state()
            self.greenhouse.heater_status = self.lamps.white.get_status()
            self.greenhouse.buzzer_state = self.buzzer.get_state()
            self.greenhouse.buzzer_status = self.buzzer.get_status()
            self.greenhouse.ain_pot_raw = self.analog.pot.get_value()
            self.greenhouse.ain_light_raw =  self.analog.light.get_value()
            self.greenhouse.ain_aux_raw =  self.analog.aux.get_value()

            self.greenhouse.temp_inside_C = self.temperature.get_inside_temp_C()
            self.greenhouse.temp_inside_F = self.temperature.get_inside_temp_F()
            self.greenhouse.temp_outside_C = self.temperature.get_outside_temp_C()
            self.greenhouse.temp_outside_F = self.temperature.get_outside_temp_F()
            
            self.greenhouse.last_update = datetime.datetime.now().replace(microsecond=0).isoformat(' ')
            
            self._make_message()
            

        except Exception as e:
            self.greenhouse.message = "Updata exception: %s" % e.args[0]
            
    def _make_message(self):
        m = ""
        gh = self.greenhouse
        if  gh.temp_outside_C > gh.temp_inside_C: 
            m = "Warning: The temperature outside the greenhouse is higher than the internal temperature."
        elif gh.servo_status == "CLOSED" and gh.fan_status == "ON":
            m = "Error: The fan is activated by louvers are closed."
        else:
            m = "Greenhouse status is normal."

        gh.message = m


class IoTGreenhouse(object):
    """IoT Greenhouse data object"""
    #house IDs
    name = None
    house_id = None
    group_id = None
    house_number = "00"
    version = None

    #house state
    led_red_state = None
    led_red_status = None
    led_white_state = None
    led_white_status = None
    led_dual_state = None
    led_dual_status = None
    switch_pb_state = None
    switch_pb_status = None
    switch_toggle_state = None
    switch_toggle_status = None
    fan_state = None
    fan_status = None
    servo_position = None
    servo_status = None
    heater_state = None
    heater_status = None
    buzzer_state = None
    buzzer_status = None
    ain_pot_raw = None
    ain_light_raw = None
    ain_aux_raw = None
    temp_inside_C = None
    temp_inside_F = None
    temp_outside_C = None
    temp_outside_F = None
    last_update = None
    
    message = None

    def __init__(self): 
        #use factory method in IoTGreenhouse Service
        pass

    



from time import sleep
import pigpio

class GHSwitches(object):
    """This class GHSwtiches provides a services to control the both IoT Greenhouse switches
    """

    SWITCH_ON = False
    SWITCH_OFF = True
    
    push_button = None
    toggle = None
    
    def __init__(self, pi, pb_switch_gpio, toggle_switch_gpio):
        """Constructor for class GHSwitches
    
        Returns references to both push button and toggle switches

        :param pi: reference to pigpio service
        :param pb_switch_gpio: GPIO port for push button switch. Use GHgpio definitions.
        :param toggle_switch_gpio: GPIO port for toggle switch. Use GHgpio definitions.
        :returns: GHSwitches object
        """
        self.push_button = GHSwitch(pi, pb_switch_gpio)
        self.toggle = GHSwitch(pi, toggle_switch_gpio)

class GHSwitch(object):
    """This class GHSwtiches provides a services to an IoT Greenhouse switch
    """
        
    _pi = None
    _gpio = 0
    
    def __init__(self, pi, switch_gpio):
        """Constructor for class GHSwitch
        
        Returns references to switch object. Used for both push button and toggle.

        :param pi: reference to pigpio service
        :param switch_gpio: GPIO port for switch. Use GHgpio definitions.
        :returns: GHSwitch object
        """
        self._pi = pi
        self._gpio = switch_gpio
        self._pi.set_mode(self._gpio, pigpio.INPUT)        
        
    def get_state(self, debounce=False):
        #debounce
        if not debounce:
            return self._pi.read(self._gpio)
        else:
            first_read = False
            second_read = True
            debouce_delay = .01
            while first_read != second_read:
                first_read = self._pi.read(self._gpio)
                sleep(debouce_delay)
                second_read = self._pi.read(self._gpio)
            return first_read

    def wait_for_press(self):
        #Wait for on - active low
        while self.get_state():
            sleep(.5)

    def wait_for_release(self):
        #Wait for off - active low
        while not self.get_state():
            sleep(.5)

    def wait_for_press_and_release(self):
        self.wait_for_press()
        self.wait_for_release()

    def is_on(self):
        return self.get_state() == GHSwitches.SWITCH_ON

    def is_off(self):
        return self.get_state() == GHSwitches.SWITCH_OFF

    def get_status(self):
        if self.get_state() == GHSwitches.SWITCH_ON:
            return "ON"
        else:
            return "OFF"